/*
 * ForumManagerBean.java
 *
 * Forum Manager bean represent functions for
 * managing the forums
 *
 * @author: Devraj Mukherjee
 *
 * Created: 07th July 2001
 *
 */

 package freeforums.forum;

 import java.rmi.RemoteException;
 import javax.ejb.*;
 import javax.naming.*;
 import javax.rmi.*;
 import java.rmi.ServerException;
 import java.util.Collection;
 import java.util.Vector;

 import freeforums.util.AutoNumberHome;
 import freeforums.util.AutoNumber;

 import freeforums.user.ForumUser;
 import freeforums.user.ForumUserHome;
 import freeforums.user.ForumUserPK;

 import org.jdom.*;
 import java.util.List;

 public class ForumManagerBean implements SessionBean {

	 /**
	 Helper function to get the home interface of a specified Bean. This is used by
	 most of the methods in this class.
	 */
	 private Object getHome (String path) {
		 try {
			 return (new InitialContext().lookup(path));
		 }
	 	catch (NamingException e) {
			System.err.println("An exception occured " + e);
			return null;
		}
	 }

	 /**
	 Empty method body
	 */
	 public ForumManagerBean() {}
	 /**
	 Empty method body
	 */
	 public void ejbCreate() {}
	 /**
	 Empty method body
	 */
	 public void ejbRemove() {}
	 /**
	 Empty method body
	 */
	 public void ejbActivate() {}
	 /**
	 Empty method body
	 */
	 public void ejbPassivate() {}
	 /**
	 Empty method body
	 */
	 public void setSessionContext(SessionContext sc) {}

	 public Document addForum(Document forumDetails) throws RemoteException {


		 Element RootElement = forumDetails.getRootElement();

		 List forumListElement = RootElement.getChildren();
		 Element [] forumElements = (Element[])forumListElement.toArray(new Element[0]);

		 String forumName = forumElements[0].getChild("ForumName").getText();
		 String forumDescription = forumElements[0].getChild("ForumDescription").getText();

		 Element responseElement = new Element("Response");
		 String identifier = "Forum";
		 Integer id=null;

 		 try {

			 /**
			  Process of creating AutoNumbers
			  **/
			 AutoNumberHome ANHome = (AutoNumberHome) getHome("java:comp/env/ejb/AutoNumber");
			 AutoNumber AutoNumberFactory = ANHome.findByPrimaryKey(identifier);
			 id = AutoNumberFactory.getAutoNumberValue();

		 }
		 catch (Exception e) {
			 /**
			  This Key was not found in the storage so
			  create a new one with this name
			  **/
			  try {
				  AutoNumberHome ANHome = (AutoNumberHome) getHome("java:comp/env/ejb/AutoNumber");
				  AutoNumber AutoNumberFactory = ANHome.create(identifier);
				  id = AutoNumberFactory.getAutoNumberValue();
			  }
			  catch(Exception exp) {
				  /**
				   Could not create this primary key
				   **/
			   }

		  }

		  if (id==null) return (new Document(responseElement.addContent("Failed")));

		  try {

			  /**
			  Process of creating the Forum Object
			  **/

			 ForumHome home = (ForumHome) getHome("java:comp/env/ejb/Forum");
			 Forum forum = home.create(id,forumName,forumDescription);
			 return (new Document(responseElement.addContent("Done")));

		 }
		 catch(Exception e) {
			 //do nothing
			 System.out.println("Exception occured while creating Forum \n" + e);
			 return (new Document(responseElement.addContent("Failed")));
		 }
	 }

	 /*
	  * Modified version returns XML document description of Forum List
	  */

	  public Document findAll() throws RemoteException, FinderException {

		  ForumHome home = (ForumHome) getHome("java:comp/env/ejb/Forum");
		  Collection all = home.findAll();
		  Forum [] forumArray = (Forum[]) all.toArray(new Forum[0]);

		  Element rootElement = new Element("ForumList");

		  for(int counter=0; counter<forumArray.length; counter++) {

			  Forum currentForum = forumArray[counter];

			  Element ForumIDElement = new Element("ForumID").addContent((currentForum.getForumID()).toString());
			  Element ForumNameElement = new Element("ForumName").addContent(currentForum.getForumName());
			  Element ForumDescriptionElement = new Element("ForumDescription").addContent(currentForum.getForumDescription());

			  Element ForumElement = new Element("Forum");
			  ForumElement.addContent(ForumIDElement);
			  ForumElement.addContent(ForumNameElement);
			  ForumElement.addContent(ForumDescriptionElement);

			  rootElement.addContent(ForumElement);

		  }

		  return (new Document(rootElement));
	  }

	  /*
	   * Find Forum method, finds a method given it's primary key
	   */

	   public Document findForum(Document forumID) throws RemoteException, FinderException {

		   Integer forumid = new Integer(forumID.getRootElement().getText());

		   try {
			   ForumHome home = (ForumHome) getHome("java:comp/env/ejb/Forum");
			   Forum currentForum = home.findByPrimaryKey(forumid);
  			   Element ForumIDElement = new Element("ForumID").addContent((currentForum.getForumID()).toString());
			   Element ForumNameElement = new Element("ForumName").addContent(currentForum.getForumName());
			   Element ForumDescriptionElement = new Element("ForumDescription").addContent(currentForum.getForumDescription());
			   Element ForumElement = new Element("Forum");

			   ForumElement.addContent(ForumIDElement);
			   ForumElement.addContent(ForumNameElement);
			   ForumElement.addContent(ForumDescriptionElement);

			   return (new Document(ForumElement));
		   }
		   catch(Exception e) {
			   return (new Document(new Element("Response").addContent("Failed")));
		   }
	   }

	  /*
	   * Find Forum method, finds a method given it's primary key
	   */

	   private Element findForumAsElement(Document forumID) throws RemoteException, FinderException {

		   Integer forumid = new Integer(forumID.getRootElement().getText());

		   try {
			   ForumHome home = (ForumHome) getHome("java:comp/env/ejb/Forum");
			   Forum currentForum = home.findByPrimaryKey(forumid);
  			   Element ForumIDElement = new Element("ForumID").addContent((currentForum.getForumID()).toString());
			   Element ForumNameElement = new Element("ForumName").addContent(currentForum.getForumName());
			   Element ForumDescriptionElement = new Element("ForumDescription").addContent(currentForum.getForumDescription());
			   Element ForumElement = new Element("Forum");

			   ForumElement.addContent(ForumIDElement);
			   ForumElement.addContent(ForumNameElement);
			   ForumElement.addContent(ForumDescriptionElement);

			   return (ForumElement);
		   }
		   catch(Exception e) {
			   return (new Element("Response").addContent("Failed"));
		   }
	   }

	   /*
	    * Returns an Array of Forum object for a given username
	    */

      public Document findForumsForUser(Document userDetails) throws RemoteException, FinderException {

		  Element RootElement = userDetails.getRootElement();
		  List UserListElement = RootElement.getChildren();
		  Element [] UserElements = (Element[])UserListElement.toArray(new Element[0]);

		  String username = (UserElements[0].getChild("UserName").getText());
		  String superuser = (UserElements[0].getChild("SuperUser").getText());

		  boolean adminStatus = false;

		  if(superuser.equals("Yes")) adminStatus = true;

		  int ForumCounter = 0;

		  Element rootElement = new Element("ForumList");

		  try {
			  ForumUserHome home = (ForumUserHome) getHome("java:comp/env/ejb/ForumUser");
			  Collection user_forums = home.findAll();
			  ForumUser[] user_forums_array = (ForumUser[]) user_forums.toArray(new ForumUser[0]);

			  for(int counter=0; counter<user_forums_array.length; counter++) {
				  if(user_forums_array[counter].getUserName().equals(username)) {
					  Element tempForum = findForumAsElement(new Document(new Element("ForumID").addContent(user_forums_array[counter].getForumID().toString())));
					  if(adminStatus==true) {
						  if(user_forums_array[counter].isAdministrator()) {
							  rootElement.addContent(tempForum);
							  ForumCounter++;
						  }

					  }
					  else {
						  rootElement.addContent(tempForum);
						  ForumCounter++;
					  }
				  }
			  }

			  System.out.println("Found " + ForumCounter + " forums for " + username);

			  return (new Document(rootElement));
		  }
		  catch(Exception e) {
			  //something happened on the way to heaven
			  System.out.println("Exception occurred while finding forums\n" + e);
			  return (new Document(new Element("Response").addContent("Failed")));
		  }

	  }

	  /*
	   * Is user allowed to use a particular forum
	   */

      public Document isUserAllowed(Document userDetails) throws RemoteException, FinderException {

		  Element rootElement = userDetails.getRootElement();

		  Element responseElement = new Element("Response");

		  String username = rootElement.getChild("UserName").getText();
		  Integer forumID = new Integer((rootElement.getChild("ForumID")).getText());

		  try {
			  ForumUserHome home = (ForumUserHome) getHome("java:comp/env/ejb/ForumUser");
			  ForumUser isAllowed = home.findByPrimaryKey(new ForumUserPK(username, forumID));
			  if (isAllowed == null) responseElement.addContent("Failed");
			  else responseElement.addContent("Done");
		  }
		  catch(Exception e) {
			  //something happened on the way to heaven
			  System.out.println("Exception occurred while finding forums\n" + e);
			  responseElement.addContent("Failed");
		  }

		  return (new Document(responseElement));

	  }

 } // end of class file
