/*
 * MessageController.java
 *
 * JavaBean to allow the JSP pages to perform the right
 * user functions.
 *
 * @author: Devraj Mukherjee
 *
 * Includes XML Support
 *
 */

package freeforums.jspbeans;

import javax.naming.*;
import java.util.Hashtable;
import javax.rmi.PortableRemoteObject;
import java.util.Properties;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.Date;

import java.util.List;

import freeforums.message.*;
import freeforums.user.*;

import org.jdom.*;

public class MessageController extends Object {

	public MessageController() { }

	/** Bridge method to addMessage **/

	public Document addMessageToForum(Document ForumDetails) {

		try	{
			//Get the home interface for the user management bean
			MessageManagerHome home = (MessageManagerHome) new InitialContext().lookup("MessageManagerBean");
			// Create the manager object
			MessageManager manager = home.create();
			return manager.addMessage(ForumDetails);
		}
 		catch(Exception e) {
			// do nothing much
			return (new Document(new Element("Response").addContent("Failed")));
		}

	}

	public Document markReadMessage(Document MessageDetails) {

		try	{
			//Get the home interface for the user management bean
			MessageManagerHome home = (MessageManagerHome) new InitialContext().lookup("MessageManagerBean");
			// Create the manager object
			MessageManager manager = home.create();
			return manager.markReadMessage(MessageDetails);
		}
 		catch(Exception e) {
			// do nothing much
			return (new Document(new Element("Response").addContent("Failed")));
		}

	}

	public Document isReadMessage(Document MessageDetails) {

		try	{
			//Get the home interface for the user management bean
			MessageManagerHome home = (MessageManagerHome) new InitialContext().lookup("MessageManagerBean");
			// Create the manager object
			MessageManager manager = home.create();
			return manager.isReadMessage(MessageDetails);
		}
 		catch(Exception e) {
			// do nothing much
			return (new Document(new Element("Response").addContent("Failed")));
		}

	}

	public Document getMessage(Document MessageDetails) {

		try	{
			//Get the home interface for the user management bean
			MessageManagerHome home = (MessageManagerHome) new InitialContext().lookup("MessageManagerBean");
			// Create the manager object
			MessageManager manager = home.create();
			return manager.getMessage(MessageDetails);
		}
 		catch(Exception e) {
			// do nothing much
			return (new Document(new Element("Response").addContent("Failed")));
		}

	}

	//////////////////////////////////////////////////////////////////////////////////////////////////
	//
	//
	// The following part of this program is the complex code that produces the threaded list of
	// messages. The code has been adopted from the previous versions of the same project.
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////////////////

	private int depth;

	/**
	 *
	 * Given a Parent Message ID this will count the number of submessages
	 *
	 **/

	private int countSubMessages(Element MessageElement, Document MessageList) {

		String ParentMessageID = MessageElement.getChild("MessageID").getText();

		List MessageListElement = MessageList.getRootElement().getChildren();
		Element [] MessageElements = (Element[])MessageListElement.toArray(new Element[0]);

		try {
			int countMessages=0;
			for(int cntr=0; cntr<MessageElements.length; cntr++)
			if(MessageElements[cntr].getChild("ReplyToID").getText().equals(ParentMessageID)) countMessages++;
			return countMessages;
		}
		catch(Exception e) {
			System.out.println("countSubMessage() failed\n" +e);
			return 0;
		}

	}

	/**
	 *
	 * Given a Message Element this method will format it as HTML and return the Element
	 *
	 * This is done so that no web specific output is generated in the Entity or Session Beans
	 *
	 * This Class is a web specific class so we can generate some HTML here.
	 *
	 **/

	 private Element formatMessage(Element MessageElement, String UserName) {

		 Element RowElement = new Element("tr");

		 String MessageID   = MessageElement.getChild("MessageID").getText();
		 String Subject     = MessageElement.getChild("Subject").getText();
		 String Author      = MessageElement.getChild("UserName").getText();
		 String MessageDate = MessageElement.getChild("MessageDate").getText();
		 String ForumID     = MessageElement.getChild("ForumID").getText();

		 // Check if message is read

		 Element ReadMessageStatus = new Element("ReadMessage");
		 ReadMessageStatus.addContent(new Element("UserName").addContent(UserName));
		 ReadMessageStatus.addContent(new Element("MessageID").addContent(MessageID));

		 Document ReadStatusResult = isReadMessage(new Document(ReadMessageStatus));

		 // Make HTML tags

		 RowElement.addContent(new Element("td").addContent(MessageDate));

		 Element ReadMessage = new Element("A");
		 ReadMessage.addAttribute("HREF","read_message.jsp?message_id=" + MessageID + "&forum_id=" + ForumID);
		 ReadMessage.addContent(Subject);

		 Element ImageTag = new Element("IMG");
		 ImageTag.addAttribute("SRC","../images/spacer.gif");
		 ImageTag.addAttribute("WIDTH",""+depth);
		 ImageTag.addAttribute("HEIGHT","1");

		 Element SubjectCol = new Element("td");
		 SubjectCol.addContent(ImageTag);

		 if(ReadStatusResult.getRootElement().getText().equals("UnRead")) {

			 Element Bold = new Element("B");
			 Bold.addContent(ReadMessage);
			 SubjectCol.addContent(Bold);

		 }
		 else SubjectCol.addContent(ReadMessage);

		 RowElement.addContent(SubjectCol);

		 RowElement.addContent(new Element("td").addContent(Author));

		 Element PostMessage = new Element("A");
		 PostMessage.addAttribute("HREF","post_message.jsp?reply_to=" + MessageID + "&forum_id=" + ForumID);
		 PostMessage.addContent("Reply");

		 RowElement.addContent(new Element("td").addContent(PostMessage));

		 return RowElement;

	 }

	/**
	 *
	 * Parent method to generate the HTML
	 *
	 **/

	public Document getMessagesForForum(Document ForumUserDetails) {

		Element MessageTableElement = new Element("table");
		MessageTableElement.addAttribute("width","100%");
		MessageTableElement.addAttribute("cellpadding","0");
		MessageTableElement.addAttribute("cellspacing","0");

		Element HeaderRow = new Element("tr");
		HeaderRow.addAttribute("bgcolor","#FFFFCC");

		Element Column1 = new Element("td");
		Column1.addAttribute("width","10%");
		Column1.addContent("Date");
		HeaderRow.addContent(Column1);

		Element Column2 = new Element("td");
		Column2.addContent("Subject");
		HeaderRow.addContent(Column2);

		Element Column3 = new Element("td");
		Column3.addAttribute("width","10%");
		Column3.addContent("Author");
		HeaderRow.addContent(Column3);

		Element Column4 = new Element("td");
		Column4.addAttribute("width","10%");
		Column4.addContent("Reply");
		HeaderRow.addContent(Column4);

		MessageTableElement.addContent(HeaderRow);

		String ForumID  = ForumUserDetails.getRootElement().getChild("ForumID").getText();
		String UserName = ForumUserDetails.getRootElement().getChild("UserName").getText();

		try {
			//Get the home interface for the user management bean
			MessageManagerHome home = (MessageManagerHome) new InitialContext().lookup("MessageManagerBean");
			// Create the manager object
			MessageManager manager = home.create();
			//Get the array
			Document MessageListDocument = manager.findMessagesForForum(new Document(new Element("ForumID").addContent(ForumID)));

			List MessageListElement = MessageListDocument.getRootElement().getChildren();
			Element [] MessageElements = (Element[])MessageListElement.toArray(new Element[0]);

			for(int cntr=0; cntr<MessageElements.length; cntr++) {
				Element currentMessage = MessageElements[cntr];
				if(currentMessage.getChild("ReplyToID").getText().equals("-1")) {
					depth=0;
					MessageTableElement.addContent(formatMessage(currentMessage, UserName));
					listSubMessages(MessageTableElement,MessageListDocument,currentMessage.getChild("MessageID").getText(),UserName);
				}
			}

			return (new Document(MessageTableElement));
		}
		catch(Exception e) {
			//something went wrong while making the HTML
			//quite unlikely that we would get this exception
			System.out.println("getMessageForForum failed \n" + e);
			return (new Document(new Element("Response").addContent("Failed")));
		}

	} // end method

	/**
	 *
	 * Converts the messages to HTML format
	 * This method is a helper method
	 *
	 **/

	private void listSubMessages(Element MessageTableElement, Document MessageListDocument, String ParentMessageID, String UserName) {

		List MessageListElement = MessageListDocument.getRootElement().getChildren();
		Element [] MessageElements = (Element[])MessageListElement.toArray(new Element[0]);

		try {
			Element currentMessage=null;
			int countMessages=0,messageIndex=0;
			depth+=8;

			for(int cntr=0; cntr<MessageElements.length; cntr++) {
				currentMessage = MessageElements[cntr];
				if(currentMessage.getChild("ReplyToID").getText().equals(ParentMessageID)) {
					countMessages++;
					messageIndex=cntr;
				}
			}

			if (countMessages==0) return;
			if (countMessages==1) {
				currentMessage=MessageElements[messageIndex];
				MessageTableElement.addContent(formatMessage(currentMessage,UserName));
				if(countSubMessages(currentMessage, MessageListDocument)>0) listSubMessages(MessageTableElement, MessageListDocument, currentMessage.getChild("MessageID").getText(), UserName);
				depth-=8;
				return;
			}
			///////////////////////////////////
			// If more than one message then //
			///////////////////////////////////
			for(int cntr=0; cntr<MessageElements.length; cntr++) {
				currentMessage=MessageElements[cntr];
				if(currentMessage.getChild("ReplyToID").getText().equals(ParentMessageID)) {
					MessageTableElement.addContent(formatMessage(currentMessage,UserName));
					// List sub messages for this reply message
					if(countSubMessages(currentMessage, MessageListDocument)>0)
					listSubMessages(MessageTableElement, MessageListDocument, currentMessage.getChild("MessageID").getText(), UserName);
				}
			}
			///////////////////////////////
			// End more than one message //
			///////////////////////////////
			depth-=8;
			return;
		}
		catch(Exception e) {
			//Something wrong happened above, quite unlikely.
			System.out.println("listSubMessages() failed \n " + e);
			return;
		}
	}

} // end class file