/*
 * MessageManagerBean.java
 *
 * Message Manager bean represent functions for
 * managing the forums
 *
 * @author: Devraj Mukherjee
 *
 * Created: 07th July 2001
 *
 */

 package freeforums.message;

 import java.rmi.RemoteException;
 import javax.ejb.*;
 import javax.naming.*;
 import javax.rmi.*;
 import java.rmi.ServerException;
 import java.util.Date;
 import java.util.Collection;
 import java.util.Vector;

 import freeforums.user.*;
 import freeforums.util.*;

 import org.jdom.*;
 import java.util.List;

 public class MessageManagerBean implements SessionBean {

	 /**
	 Helper function to get the home interface of a specified Bean. This is used by
	 most of the methods in this class.
	 */
	 private Object getHome (String path) {
		 try {
			 return (new InitialContext().lookup(path));
		 }
	 	catch (NamingException e) {
			System.err.println("An exception occured " + e);
			return null;
		}
	 }

	 /**
	 Empty method body
	 */
	 public MessageManagerBean() {}
	 /**
	 Empty method body
	 */
	 public void ejbCreate() {}
	 /**
	 Empty method body
	 */
	 public void ejbRemove() {}
	 /**
	 Empty method body
	 */
	 public void ejbActivate() {}
	 /**
	 Empty method body
	 */
	 public void ejbPassivate() {}
	 /**
	 Empty method body
	 */
	 public void setSessionContext(SessionContext sc) {}

	 /**
	  Gets a message given it's message id
	  **/

	 public Document getMessage(Document message) throws RemoteException, FinderException {

		 Integer messageID = new Integer(message.getRootElement().getText());

		 try {
			 MessageHome home = (MessageHome) getHome("java:comp/env/ejb/Message");
			 Message existingMessage = home.findByPrimaryKey(messageID);

			 Date MessageDate = existingMessage.getMessageDate();
			 String DisplayDate = MessageDate.getDate() + "/" + MessageDate.getMonth() + "/" + (1900 + MessageDate.getYear());

			 Element MessageElement = new Element("Message");

			 MessageElement.addContent(new Element("MessageID").addContent(existingMessage.getMessageID().toString()));
			 MessageElement.addContent(new Element("Subject").addContent(existingMessage.getSubject()));
			 MessageElement.addContent(new Element("MessageBody").addContent(existingMessage.getDescription()));
			 MessageElement.addContent(new Element("MessageDate").addContent(DisplayDate));
			 MessageElement.addContent(new Element("UserName").addContent(existingMessage.getUserName()));
			 MessageElement.addContent(new Element("ForumID").addContent(existingMessage.getForumID().toString()));
			 MessageElement.addContent(new Element("ReplyToID").addContent(existingMessage.getReplyToID().toString()));

			 Element RootElement = new Element("MessageList");
			 RootElement.addContent(MessageElement);

			 return (new Document(RootElement));
		 }
		 catch(Exception e) {
			 return (new Document(new Element("Response").addContent("Failed")));
		 }
	 }

	 /**
	  Adds a new message to the collection
	  **/

	 public Document addMessage(Document MessageInformation) throws RemoteException, FinderException {

		 Element RootElement = MessageInformation.getRootElement();

		 List MessageListElement = RootElement.getChildren();
		 Element [] MessageElements = (Element[])MessageListElement.toArray(new Element[0]);

		 String subject = MessageElements[0].getChild("Subject").getText();
		 String description = MessageElements[0].getChild("MessageBody").getText();
		 Integer forumID = new Integer(MessageElements[0].getChild("ForumID").getText());
		 String userName = MessageElements[0].getChild("UserName").getText();
		 Integer replyToID = new Integer(MessageElements[0].getChild("ReplyToID").getText());


		 String identifier = "Message";
		 Integer id=null;

 		 try {

			 /**
			  Process of creating AutoNumbers
			  **/
			 AutoNumberHome ANHome = (AutoNumberHome) getHome("java:comp/env/ejb/AutoNumber");
			 AutoNumber AutoNumberFactory = ANHome.findByPrimaryKey(identifier);
			 id = AutoNumberFactory.getAutoNumberValue();

		 }
		 catch (Exception e) {
			 /**
			  This Key was not found in the storage so
			  create a new one with this name
			  **/
			  try {
				  AutoNumberHome ANHome = (AutoNumberHome) getHome("java:comp/env/ejb/AutoNumber");
				  AutoNumber AutoNumberFactory = ANHome.create(identifier);
				  id = AutoNumberFactory.getAutoNumberValue();
			  }
			  catch(Exception exp) {
				  /**
				   Could not create this primary key
				   **/
			   }

		  }

		  if (id==null) return (new Document(new Element("Response").addContent("Failed")));

		  try {

			  /**
			  Process of creating the Forum Object
			  **/

			 MessageHome home = (MessageHome) getHome("java:comp/env/ejb/Message");
			 Message tempMessage = home.create(id,subject,description,forumID,userName);
			 tempMessage.setReplyToID(replyToID);
			 tempMessage.setMessageDate(new Date());
			 return (new Document(new Element("Response").addContent("Done")));

		 }
		 catch(Exception e) {
			 //do nothing
			 System.out.println("Exception occured while creating Forum \n" + e);
			 return (new Document(new Element("Response").addContent("Failed")));
		 }

	 } // end add message method

	/**
	 Marks a message to be read
	 **/

	public Document markReadMessage(Document MessageInfo) throws RemoteException, FinderException {

		Element rootElement = MessageInfo.getRootElement();
		String userName = rootElement.getChild("UserName").getText();
		Integer messageID = new Integer(rootElement.getChild("MessageID").getText());

		try {
			ReadMessageHome home = (ReadMessageHome) getHome("java:comp/env/ejb/ReadMessage");
			ReadMessage tempReadMessage = home.create(new ReadMessagePK(userName, messageID));
			return (new Document(new Element("Response").addContent("Done")));
		 }
		 catch(Exception e) {
			 //do nothing
			 System.out.println("Exception while marking message read \n" + e);
			return (new Document(new Element("Response").addContent("Failed")));
		 }
	} // end method

	/**
	 Checks to see if a message is read by a user
	 **/

	public Document isReadMessage(Document MessageInfo) throws RemoteException, FinderException {

		Element rootElement = MessageInfo.getRootElement();
		String userName = rootElement.getChild("UserName").getText();
		Integer messageID = new Integer(rootElement.getChild("MessageID").getText());

		try {
			ReadMessageHome home = (ReadMessageHome) getHome("java:comp/env/ejb/ReadMessage");
			ReadMessage tempReadMessage = home.findByPrimaryKey(new ReadMessagePK(userName, messageID));
			if(tempReadMessage==null) return (new Document(new Element("Response").addContent("Read")));
			else return (new Document(new Element("Response").addContent("UnRead")));
		 }
		 catch(Exception e) {
			 //do nothing
			 return (new Document(new Element("Response").addContent("UnRead")));
		 }
	} // end method


	 /**
	  finds messages for a forum id
	  **/

	 public Document findMessagesForForum(Document ForumID) throws RemoteException, FinderException {

		  Integer forumID = new Integer(ForumID.getRootElement().getText());

		  Element rootElement = new Element("MessageList");

		  try {
			  MessageHome home = (MessageHome) getHome("java:comp/env/ejb/Message");
			  Collection messages = home.findAll();
			  Message[] messageArray = (Message[]) messages.toArray(new Message[0]);

			  for(int counter=0; counter<messageArray.length; counter++) {

				  if((messageArray[counter].getForumID()).equals(forumID)) {

					  // Convert the message to a XML document

					 Message existingMessage = messageArray[counter];

					 Date MessageDate = existingMessage.getMessageDate();

					 String DisplayDate = MessageDate.getDate() + "/" + MessageDate.getMonth() + "/" + (1900 + MessageDate.getYear());

					 Element MessageElement = new Element("Message");
					 MessageElement.addContent(new Element("MessageID").addContent(existingMessage.getMessageID().toString()));
					 MessageElement.addContent(new Element("Subject").addContent(existingMessage.getSubject()));
					 MessageElement.addContent(new Element("MessageBody").addContent(existingMessage.getDescription()));
					 MessageElement.addContent(new Element("MessageDate").addContent(DisplayDate));
					 MessageElement.addContent(new Element("UserName").addContent(existingMessage.getUserName()));
					 MessageElement.addContent(new Element("ForumID").addContent(existingMessage.getForumID().toString()));
					 MessageElement.addContent(new Element("ReplyToID").addContent(existingMessage.getReplyToID().toString()));

					 rootElement.addContent(MessageElement);

				  }

			  }

			  return (new Document(rootElement));
		  }
		  catch(Exception e) {
			  //something happened on the way to heaven
			  System.out.println("Exception occurred while finding messages\n" + e);
			  return (new Document(new Element("Response").addContent("Done")));
		  }
	} // end method

 } // end class file