/*
 * UserManagerBean.java
 *
 * Session bean that represents the User Management
 * facilities for the FreeForums System.
 *
 * @author: Devraj Mukherjee
 *
 * Created: 24th June 2001
 *
 */

 package freeforums.user;

 import java.rmi.RemoteException;
 import javax.ejb.*;
 import javax.naming.*;
 import javax.rmi.*;
 import java.rmi.ServerException;
 import java.util.Collection;
 import java.util.List;

 import org.jdom.*;

 public class UserManagerBean implements SessionBean {

	 /**
	 Helper function to get the home interface of a specified Bean. This is used by
	 most of the methods in this class.
	 */
	 private Object getHome (String path) {
		 try {
			 return (new InitialContext().lookup(path));
		 }
	 	catch (NamingException e) {
			System.err.println("An exception occured " + e);
			return null;
		}
	 }

	 /**
	 Empty method body
	 */
	 public UserManagerBean() {}
	 /**
	 Empty method body
	 */
	 public void ejbCreate() {}
	 /**
	 Empty method body
	 */
	 public void ejbRemove() {}
	 /**
	 Empty method body
	 */
	 public void ejbActivate() {}
	 /**
	 Empty method body
	 */
	 public void ejbPassivate() {}
	 /**
	 Empty method body
	 */
	 public void setSessionContext(SessionContext sc) {}

	 public Document modifyUser(Document userDetails) throws RemoteException {

		 /*****
		  * Parse XML data
		  *
		  * This modifyUser method now supports multiple users
		  *****/

		 String userName,password,realName,emailAddress;
		 boolean superUser;

		 Element userListElement = userDetails.getRootElement();
		 Element responseElement = new Element("Response");

		 // Extract the User Elements

		 List userElementList = userListElement.getChildren();
		 Element [] userElements = (Element[])userElementList.toArray(new Element[0]);

		 int counter=0;

		 UserHome home = (UserHome) getHome("java:comp/env/ejb/User");

		 userName = (userElements[counter].getChild("UserName")).getText();
		 password = (userElements[counter].getChild("Password")).getText();
		 realName = (userElements[counter].getChild("RealName")).getText();
		 emailAddress = (userElements[counter].getChild("EmailAddress")).getText();

		 if((userElements[counter].getChild("SuperUser")).getText().equals("Yes")) superUser=true;
		 else superUser=false;

		 try {
			 User existingUser = home.findByPrimaryKey(userName);
			 existingUser.setPassword(password);
			 existingUser.setRealName(realName);
			 existingUser.setEmailAddress(emailAddress);
			 existingUser.setSuperUser(superUser);
			 responseElement.addContent("Done");
		 }
		 catch(Exception e) {
			 // error occured
			 responseElement.addContent("Failed");
		 }

		 return (new Document(responseElement));
	 }

	 /**************
	  *
	  * Method to add new usrs
	  *
	  **************/

	 public Document addUser(Document userDetails) throws RemoteException {

		 /*****
		  * Parse XML data
		  *****/

		 String userName,password,realName,emailAddress;
		 boolean superUser;

		 Element userListElement = userDetails.getRootElement();
		 Element responseElement = new Element("Response");

		 // Extract the User Elements

		 List userElementList = userListElement.getChildren();
		 Element [] userElements = (Element[])userElementList.toArray(new Element[0]);

		 UserHome home = (UserHome) getHome("java:comp/env/ejb/User");

		 int counter=0;

		 // Get all the details off the user element

		 userName = (userElements[counter].getChild("UserName")).getText();
		 password = (userElements[counter].getChild("Password")).getText();
		 realName = (userElements[counter].getChild("RealName")).getText();
		 emailAddress = (userElements[counter].getChild("EmailAddress")).getText();

		 if((userElements[counter].getChild("SuperUser")).getText().equals("Yes")) superUser=true;
		 else superUser=false;

		 try {
			 User existingUser = home.findByPrimaryKey(userName);
			 String oldUserName = existingUser.getUserName(); // Force loading of data here
			 // If we get here, the user is already in the collection
			 System.out.println("Could not complete request \n User exists " + oldUserName);
			 responseElement.addContent("Duplicate");
		}
		// The database does not have an User with the same username
		// so it is now safe to try and add a new user

		catch (Exception e) {
			System.out.println("Added new user with the userName = " + userName);
			try {
				User user = home.create(userName);
				user.setPassword(password);
				user.setRealName(realName);
				user.setEmailAddress(emailAddress);
				user.setSuperUser(superUser);
				responseElement.addContent("Done");
			}
			catch (Exception ex) {
				responseElement.addContent("Failed");
			}
		}

		return (new Document(responseElement));

	 }

	 /*
	  * Return a user object
	  */

      public Document getUser(Document userDocument) throws RemoteException {

		 Element rootElement = userDocument.getRootElement();
		 String username = rootElement.getText();

		 UserHome home = (UserHome) getHome("java:comp/env/ejb/User");
		 try {
			User existingUser = home.findByPrimaryKey(username);
  		    Element user = new Element("User");

			user.addContent(new Element("UserName").addContent(existingUser.getUserName()));
			user.addContent(new Element("Password").addContent(existingUser.getPassword()));
			user.addContent(new Element("RealName").addContent(existingUser.getRealName()));
			user.addContent(new Element("EmailAddress").addContent(existingUser.getEmailAddress()));

			if(existingUser.isSuperUser()) user.addContent(new Element("SuperUser").addContent("Yes"));
			else user.addContent(new Element("SuperUser").addContent("No"));

			Element rootResponseElement = new Element("UserList");
			rootResponseElement.addContent(user);

			return (new Document(rootResponseElement));

		 }
		 catch(Exception e) {
			 Element rootResponseElement = new Element("Response");
			 rootResponseElement.addContent("Not Found");
			 return (new Document(rootResponseElement));
		 }
	 }

	 /*
	  * Login process for a user
	  *
	  * Check for the existance of the user account and then check the password.
	  *
	  */

	 public Document loginUser(Document loginDocument) throws RemoteException {

		 UserHome home = (UserHome) getHome("java:comp/env/ejb/User");

		 Element userListElement = loginDocument.getRootElement();
		 List userElementList = userListElement.getChildren();
		 Element [] userElements = (Element[])userElementList.toArray(new Element[0]);

		 String userName = userElements[0].getChild("UserName").getText();
		 String password = userElements[0].getChild("Password").getText();

		 Element responseElement = new Element("Response");

		 try {
			 User existingUser = home.findByPrimaryKey(userName); // find the user
			 if(existingUser.getPassword().equals(password))
			 return getUser(new Document(new Element("UserName").addContent(userName)));
			 else responseElement.addContent("Failed");
		 }
		 catch(Exception e) {
			 System.out.println("Could not complete request \n" + e);
			 responseElement.addContent("Failed");
		 }

		 return (new Document(responseElement));

	 }

	 /*
	  * add a user to a forum
	  *
	  * if the user already is subscribed to that forum then
	  * this method will return false, else it will return true.
	  */

	 public Document addUserToForum(Document userDetails) throws RemoteException {

		 Element rootElement = userDetails.getRootElement();
		 Element responseElement = new Element("Response");

		 List userElementList = rootElement.getChildren();
		 Element [] userElements = (Element[])userElementList.toArray(new Element[0]);

		 String userName = userElements[0].getChild("UserName").getText();
		 Integer forumID = new Integer(userElements[0].getChild("ForumID").getText());
		 String adminstatus = userElements[0].getChild("Administrator").getText();

		 boolean administrator = false;

		 if(adminstatus.equals("Yes")) administrator = true;

		 ForumUserHome home = (ForumUserHome) getHome("java:comp/env/ejb/ForumUser");

		 try {
			 ForumUser existingForumUser = home.findByPrimaryKey(new ForumUserPK(userName, forumID));
			 String existingUserName = existingForumUser.getUserName();
			 responseElement.addContent("Done");
			 System.out.println("User " + userName + " allowed to use " + forumID);
		 }
		 catch(Exception e) {
			 try {
				 ForumUser tempForumUser = home.create(new ForumUserPK(userName, forumID));
				 tempForumUser.setAdministrator(administrator);
				 responseElement.addContent("Done");
				 System.out.println("User " + userName + " allowed to use " + forumID);
			 }
			 catch (Exception ex) {
				 responseElement.addContent("Failed");
				 System.out.println("Failed - User " + userName + " allowed to use " + forumID);
			 }
		 }

		 return (new Document(responseElement));
	 }

	 /***
	  * remove a user from a forum
	  ***/

	 public Document removeUserFromForum(Document userDetails) throws RemoteException {

		 Element rootElement = userDetails.getRootElement();
		 Element responseElement = new Element("Response");

		 List userElementList = rootElement.getChildren();
		 Element [] userElements = (Element[])userElementList.toArray(new Element[0]);

		 String userName = userElements[0].getChild("UserName").getText();
		 Integer forumID = new Integer(userElements[0].getChild("ForumID").getText());

		 try {
			 ForumUserHome home = (ForumUserHome) getHome("java:comp/env/ejb/ForumUser");
			 ForumUser existingForumUser = home.findByPrimaryKey(new ForumUserPK(userName, forumID));
			 existingForumUser.remove();
			 responseElement.addContent("Done");
		 }
		 catch(Exception e) {
			 responseElement.addContent("Failed");
		 }

		 return (new Document(responseElement));
	 }


	 /*****
	  *
	  * findAll()
	  *
	  * Creates a XML representation of the User List based on User.dtd
	  *
	  *****/

	  public Document findAll() throws RemoteException, FinderException {

		  UserHome home = (UserHome) getHome("java:comp/env/ejb/User");
		  Collection all = home.findAll();
		  User [] userArray = (User[]) all.toArray(new User[0]);
		  DocType dtd = new DocType("UserList","http://devraj.org/dtd/freeforums/User.dtd");

		  Element root = new Element("UserList");

		  for(int counter=0; counter<userArray.length; counter++) {

			  Element user = new Element("User");
			  User currentUser = userArray[counter];

			  user.addContent(new Element("UserName").addContent(currentUser.getUserName()));
			  user.addContent(new Element("Password").addContent(currentUser.getPassword()));
			  user.addContent(new Element("RealName").addContent(currentUser.getRealName()));
			  user.addContent(new Element("EmailAddress").addContent(currentUser.getEmailAddress()));

			  if(currentUser.isSuperUser()) user.addContent(new Element("SuperUser").addContent("Yes"));
			  else user.addContent(new Element("SuperUser").addContent("No"));

			  root.addContent(user);

		  }

		  return (new Document(root,dtd));
	  }


 } // this bracket ends the class file.